package Tests::verify_email;

use Tests::Base;
@ISA = qw( Tests::Base );

use IO::Socket::INET;

%AUTOFIELDS = (
	smtp_code => undef,
	smtp_message => undef
);

sub execute
{
	my $self = shift;
	my $address = $self -> opt( 'address' ) || $self -> host_addr();
	my $server = $self -> opt( 'server' );
	my $port = $self -> opt( 'port' );

	if( ! $address )
	{
		$self -> message( "Address required" );
		return( 1 );
	}

	if( ! $server )
	{
		$self -> message( "Server required" );
		return( 1 );
	}

	my $s = IO::Socket::INET -> new( PeerAddr => $server,
		PeerPort => $port || '25' );

	if( ! $s )
	{
		$self -> message( "Error creating socket: $!" );
		return( 1 );
	}

	# Do proto
	return( 1 ) unless $self -> read_verify_smtp( $s, '220', 'Error on initial read' );

	$s -> print( "helo sentinel\r\n" );
	return( 1 ) unless $self -> read_verify_smtp( $s, '250', 'Error saying helo' );

	$s -> print( "mail from: sentinel\@localhost\r\n" );
	return( 1 ) unless $self -> read_verify_smtp( $s, '250', 'Error on mail from' );

	$s -> print( "rcpt to: $address\r\n" );
	return( 1 ) unless $self -> read_verify_smtp( $s, '250', 'Error on rcpt to' );

	$s -> print( "quit\r\n" );
	$s -> close();

	# Made out OK
	return( 0 );
}

sub read_verify_smtp
{
	# Wraps the two functions below into one call
	my $self = shift;
	my $s = shift;
	my $code = shift;
	my $msg = shift;

	$self -> read_smtp( $s );
	return( $self -> verify_smtp( $code, $msg ) );
}

sub read_smtp
{
	my $self = shift;
	my $s = shift;

	my( $code, $message );

	my $line = $s -> getline();

	$line =~ s/\r//g;
	chomp( $line );

	( $code, $message ) = $line =~ /^(\d+)\s+(.*)/;

	$self -> smtp_code( $code );
	$self -> smtp_message( $message );

	if( ! $code )
	{
		$self -> message( "Error parsing SMTP line: $line" );
	}
}

sub verify_smtp
{
	# Takes the code to verify, and message to send, returns 0 on nomatch
	my $self = shift;
	my $code = shift;
	my $msg = shift;

	if( $self -> smtp_code() != $code )
	{
		$self -> message( "$msg: " . $self -> smtp_message() );
		return( 0 );
	}

	return( 1 );
}

1;

__END__

=head1 NAME

Tests::verify_email - verify an email account exists on a mail server

=head1 SYNOPSIS

	<verify_email address="user@host" server="server.address.com"/>

=head1 DESCRIPTION

Initiates an SMTP conversation with server to verify if address exists.  Fails
if it does not get 250 OK after sending the rcpt to: line.  It uses he rcpt to:
method because most servers have EXPN and VRFY turned off.  It does not generate
a real email, it will stop after the address is verified.

The server should be the address where the final recipient is located because if
if the email is queued for a third party it may bounce and the plugin won't catch
that, only if the server denied delivery immediately would the error be cought,
so it must be local to server.

=head2 address

The email address to verify

=head2 server

The mail server to verify the address on, defaults to the test host

=head1 SEE ALSO

sentinel, probe, librarian, sentinel.conf

=head1 AUTHOR

Steve Slaven - http://hoopajoo.net
